#include <scl1.h>
#include <stdlib.h>

/*                       SCL1.LIB's Keyboard Functions Demo

	To run this demo inside QuickC environment:

		Copy SCL1.H into your INCLUDE subdirectory.
		Load SCL1 Quick Library.
		Turn Screen Swapping On (QC's Debug Menu).
		Compile it (memory output option and debug option on if you
					want to step through).


		To Load SCL1 Quick Library:

			qc /l scl1

	File SCL1.QLB must be located in the path specified in the LIB
	environment variable directory or in your default drive/path


	 Functions declaration */

int helpf(struct InputFields *x);   /* Sample Fields help function */
int test(struct InputFields *p);    /* Sample Fields validation function */


/* Program data */

char Color1=GREEN_BLACK,Color2=RED_WHITE;

char Key1[]="Ŀ\n"
            " Alt:    Not Pressed  Insert:   Off  Num Lock:   Off  L. Shift:Not Pressed \n"
            "Ĵ\n"
            " Control:Not Pressed  Caps Lock:Off  Scroll Lock:Off  R. Shift:Not Pressed \n"
            "";

char Key2[]="Press any other key to exit.........";

char Key16[]="ͻ\n"
			 "            SAMPLE DATA ENTRY SCREEN USING THE FIELDS FUNCTION                \n"
			 "͹";

char Key17[]="                                                                             \n";

char Key18[]="͹\n"
			 " Use the Arrow Keys to move between fields. Some fields are set to accept only\n"
			 " some kind of data. The Zip field accepts integers, the transaction number re-\n"
			 " quires an integer number greater than 1000.  Character fields accept special \n"
			 " spanish language symbols, try ALT-A, E, I, O, U, N.  Try entering the data in\n"
			 " all the fields also try entering a number smaller than 1000 in the Transac-  \n";

char Key19[]=" tion Number field.                                                           \n"
			 "͹\n"
			 "    F1 - Help                                             F10 - Exit          \n"
			 "ͼ";
char Key20[]="[ HELP ]\n\n Help Screens  can be  defined for\n each field  defined in the Fields\n"
			 " Structure. When the assigned Help\n Key is pressed, in this case  F1,\n"
			 " control is passed to  the corres-\n ponding help function. It is easy\n"
			 " to provide context sensitive help\n for each of the fields.\n\n\n"
			 "     PRESS ANY KEY TO CONTINUE";

/* Buffers used for the field, getstring and dialog functions. Buffers size
   equals maximum number of characters + 1 */

char buffer1[29],buffer2[29],buffer3[29],buffer4[29],buffer5[13];
char buffer6[11],buffer7[11],buffer8[11],buffer9[9],buffer10[11],buffer11[32],buffer12[32];

/* Structure definition for sample Fields function. Each record=row & column
	prompt position, pointer to prompt, row & column entry field position,
	pointer to buffer, maximum number of characters to accept, type of
	characters to accept, pointer to optional help funtion, pointer to
	optional validation funtion */

struct InputFields data[]={
   4,3,"Name:",4,11,buffer1,28,any,helpf,0,
   6,3,"Address:",6,11,buffer2,28,any,helpf,0,
   8,3,"City:",8,11,buffer3,28,any,helpf,0,
   10,3,"State:",10,11,buffer4,28,any,helpf,0,
   12,3,"Zip:",12,11,buffer5,12,digit,helpf,0,
   4,44,"Customer Number:",4,60,buffer6,10,digit,helpf,0,
   6,44,"Amount Due:   $",6,60,buffer7,10,number,helpf,0,
   8,44,"Credit Limit: $",8,60,buffer8,10,number,helpf,0,
   10,44,"Date:",10,49,buffer9,8,any,helpf,0,
   12,44,"Transaction Number:",12,64,buffer10,10,number,helpf,test};

/* We'll add a new error to ErrorBox. It will be used by the sample
   field validation function */

struct ErrorMess em[]={              /* Definition of error message */
 256,"Invalid Entry",};              /* Error 256 */

/* CODE */

main()
{
unsigned int Key;
int i;

/* Clear screen, check monitor type, turn cursor off */

Cls(Color1,CLS_ALL);
if(Video()==MONO)
	Color1=WHITE_BLACK,Color2=BLACK_WHITE;
CursorOff();

if(CheckMouse())        /* Check if mouse is present */
	{
	ResetMouse();       /* Initialize if so */
	SetMouseIsr();
	}

/* Add local error to ErrorBox */

InitUserError(em);  /* Initialize error message routine */

/* Prepare screen for Keyboard status demo */

WriteScreen(Color2,14,30,"KEYBOARD STATUS");
WriteScreen(Color1,16,0,Key1);
WriteScreen(Color1,22,19,Key2);

/* Function KeyStatus returns the keyboard status. By performing a series
   of AND operation with mask defined in SCL1.H we are able to find the
   state of ALT, CONTROL, SHIFT, etc. keys. */

do
    {
    Key=KeyStatus();
	if (Key&ALT)                                /* Check status of ALT key */
		WriteScreen(Color2,17,10,"Pressed    ");/* If > 0 key is pressed or */
	else                                        /* toggled */
        WriteScreen(Color1,17,10,"Not Pressed");
	if (Key&CTRL)                               /* Check status of CONTROL key */
        WriteScreen(Color2,19,10,"Pressed    ");
    else
        WriteScreen(Color1,19,10,"Not Pressed");
	if (Key&INSERT)                             /* Check status of INSERT key */
        WriteScreen(Color2,17,34,"On ");
    else
        WriteScreen(Color1,17,34,"Off");
	if (Key&CAPSL)                              /* Check status of CAPS LOCK key */
        WriteScreen(Color2,19,34,"On ");
    else
        WriteScreen(Color1,19,34,"Off");
	if (Key&NUML)                               /* Check status of NUM LOCK key */
        WriteScreen(Color2,17,52,"On ");
    else
        WriteScreen(Color1,17,52,"Off");
	if (Key&SCROLL)                             /* Check status of SCROLL LOCK key */
        WriteScreen(Color2,19,52,"On ");
    else
        WriteScreen(Color1,19,52,"Off");
	if (Key&LSHIFT)                             /* Check status of LEFT SHIFT key */
        WriteScreen(Color2,17,67,"Pressed    ");
    else
        WriteScreen(Color1,17,67,"Not Pressed");
	if (Key&RSHIFT)                             /* Check status of RIGHT SHIFT key */
        WriteScreen(Color2,19,67,"Pressed    ");
    else
		WriteScreen(Color1,19,67,"Not Pressed");

	/* KeyReady function returns the Scan+Ascii Code of any key in the
	   BIOS buffer or 0 if the buffer is empty. But it does not removes
	   the key from the buffer. We'll use KeyReady to see if INSERT has
	   been pressed (ALT, CONTROL, SHIFT, etc. keys are not reported by
	   the BIOS). If so, we'll use ClearKeyBuf since we don't want our loop
	   to end when the user press Insert. */

	if(KeyReady()==INS)
		ClearKeyBuf();

	/* We'll use KeyReady again to see if any key other than INSERT
	   has been pressed */

	}while(!KeyReady() || KeyReady()==INS);



		   /* GetString Demo */

ClearKeyBuf();
Cls(Color1,CLS_ALL);

 /* GetString parameters: Prompt Color, Prompt's row & column, pointer to
	prompt string, entry field's color, row & column, maximum number of
	characters to accept, type of characters to accept, pointer to buffer
	We'll accept up to 31 characters but only letters (+spanish letters and
	punctuation characters will be accepted
	Returns number od characters typed or -1 if ESCAPE was pressed to exit */

GetString(Color1,10,20,"Name:",Color2,10,28,31,espanol+punctuation,buffer11);

 /* DialogBox Demo */

Cls(Color1,CLS_ALL);

 /* DialogBox saves the screen area, draws a box and display your prompt
	Parameters: Pormpt color, pointer to prompt string, field color,
	maximum number of characters, type, pointer to buffer */

DialogBox(Color1,"Filename",Color2,31,FILESPEC,buffer12);


		   /* Fields Demo */

Box(Color1,0,0,0,15,79);                  /* Write screen messages */
WriteScreen(Color1,0,0,Key16);
for(i=3;i<15;++i)
	WriteScreen(Color1,i,0,Key17);
WriteScreen(Color1,15,0,Key18);
WriteScreen(Color1,21,0,Key19);

	/* Fields supports mouse */

if(MOUSE_FLAG)
	ShowMouse();

/* Fields parameters: Prompt color, fields color, number of fields,
   pointer to fields structure, Exit key and Help key */

Fields(Color1,Color2,10,data,F10,F1);  /* Call Fields function */

Cls(Color1,CLS_ALL);

/* GetKey waits for a key to be pressed and returns the key Scan+Ascii code
   Most key codes has been defined in SCL1.H */

WriteScreen(Color2,24,0," Press F1 to end the Demo ");
do
	{
	}while(GetKey()!=F1);

	/* Use YesNo function to display a box with a question and a two YES or
		NO mouse buttons. 1 = YES, 2 = NO
		Screen is saved and restored.
		Parameters: char Color for box and message
					char Color for selected button
					button to highlight, 1 = YES, 2 = NO
					pointer to question */

if(YesNo(Color1,Color2,2,"Exit to DOS?")==2)
	WriteScreen(Color1,10,23,"Sorry but you'll have to exit anyway!");
CursorOn();
if(MOUSE_FLAG)
	ResetMouse();
}

/* You can specify Help functions for each field. They will be called
   each time the user press the help key. They receive a pointer to
   the active field */

int helpf(struct InputFields *x)

{
char WinFlag=0,WinBuf[1036],L,C;

L=GetCurLine();
C=GetCurCol();
SetCurPos(25,0);
Window(Color2,2,2,15,38,&WinFlag,WinBuf);
Cls(Color2,2,2,15,38);
Box(Color2,0,2,2,15,38);
WriteScreen(Color2,2,3,Key20);
WaitKeyMouse();                            /* Wait for key */
Window(Color2,2,2,15,38,&WinFlag,WinBuf);  /* Restore display area */
SetCurPos(L,C);
}

/* Sample field validation function called by Fields. You can specify
   a function for each field. Theu are called each time the cursor leaves
   a field and they receive a pointer to the active field. If they return
   0 the cursor will be forced to stay in the same field */

int test(struct InputFields *p)
{
unsigned long x;         /* any integer value */
char *s;

s=p->FieldBuffer;       /* get FieldBuffer contents */
x=atol(s);              /* convert it to an integer value */
if (x<1000)             /* entry is invalid */
	{                   /* Write error message in window */
	ErrorBox(256);
	return(0);
	}
else
	return(1);          /* Entry is valid */
}
